@@ -72,6 +72,7 @@ class PreviewCameraX(PreviewCommon, CommonGestures):
         self._name_pipe = []
         self.texture_size = []
         self.rotation = 0
+        self.capture_in_progress = False
 
         # uniform case
         self.flash_state = default_flash.lower()
@@ -130,12 +131,12 @@ class PreviewCameraX(PreviewCommon, CommonGestures):
     # destroy camera
     @run_on_ui_thread
     def disconnect_camera(self):
+        self.stop_capture_video()
         self._deschedule_pipeline()
         if self._camera:
             self._camera.unbind_camera()
             self._camera = None
-        return self.texture_size
-
+                
     # configure camera
     def _configure_camera(self, start):
         self.configure_viewport()
@@ -150,8 +151,9 @@ class PreviewCameraX(PreviewCommon, CommonGestures):
 
     # Device Rotate
     def on_size(self, instance, size):
-        self.stop_capture_video()
-        self._configure_camera(False)
+        if self._camera:
+            self.stop_capture_video()
+            self._configure_camera(False)
             
     ##################################
     # Parse options
@@ -174,12 +176,12 @@ class PreviewCameraX(PreviewCommon, CommonGestures):
     # CommonGestures Touch Events
     # tap for focus
     def cg_tap(self, touch, x, y):
-        if self.enable_focus_gesture:
+        if self._camera and self.enable_focus_gesture:
             self.focus(x, y)
 
     # pinch/spread for zoom
     def cg_scale(self, touch0, touch1, scale, x, y):
-        if self.enable_zoom_gesture:
+        if self._camera and self.enable_zoom_gesture:
             self.zoom_delta(scale)
 
     ##############################
@@ -188,6 +190,7 @@ class PreviewCameraX(PreviewCommon, CommonGestures):
 
     def capture_photo(self, location = '',  subdir = '', name = ''):
         if self._camera:
+            self.capture_in_progress = True
             self._set_location(location)
             subdir = self._default_subdir_android(subdir)
             name = self._default_file_name(name, '.jpg')
@@ -197,6 +200,7 @@ class PreviewCameraX(PreviewCommon, CommonGestures):
 
     def capture_video(self, location = '', subdir = '', name = ''):
         if self._camera:
+            self.capture_in_progress = True
             self._set_location(location)
             subdir = self._default_subdir_android(subdir)
             name = self._default_file_name(name,'.mp4')
@@ -248,10 +252,9 @@ class PreviewCameraX(PreviewCommon, CommonGestures):
     
 
     # Select back, front camera
-    @run_on_ui_thread
     def select_camera(self, facing):
-        facing = facing.lower()
         if self._camera:
+            facing = facing.lower()
             if facing == 'toggle':
                 if self.facing == 'back':
                     self.facing = 'front'
@@ -261,9 +264,26 @@ class PreviewCameraX(PreviewCommon, CommonGestures):
                 self.facing = 'front'
             else:
                 self.facing = 'back'
-            self._camera.select_camera(self.facing)
-        return self.facing
 
+            # may have to wait for a capture to complete
+            if not self.capture_in_progress:
+                self.do_select_camera()
+            else:
+                self.stop_capture_video()
+                self._facing_ev = Clock.schedule_interval(
+                    self.can_select_camera, 1 / 30)
+            facing = self.facing
+        return facing
+
+    def can_select_camera(self,dt):
+        if not self.capture_in_progress:
+            self.do_select_camera()
+            Clock.unschedule(self._facing_ev)
+
+    @run_on_ui_thread
+    def do_select_camera(self):
+        self._camera.select_camera(self.facing)
+            
     # Sequence flash : off, on, auto, ...
     def flash(self):
         if self._camera:
@@ -274,7 +294,8 @@ class PreviewCameraX(PreviewCommon, CommonGestures):
             else:
                 self.flash_state = 'off'
             self.flash_state = self._camera.flash(self.flash_state)
-        return self.flash_state
+            return self.flash_state
+        return "off"
 
     # if enable_focus_gesture == True, then this is called by a tap gesture
     def focus(self, x, y):
@@ -314,13 +335,6 @@ class PreviewCameraX(PreviewCommon, CommonGestures):
         elif rotation == 270:
             translate = (long_edge - short_edge) /2
 
-        # move to canvas_callback
-        #if self.facing == 'front':
-        #    if rotation == 90 or rotation == 270:
-        #        scaley = -1
-        #    else:
-        #        scalex = -1
-
         if texture_size[0] < texture_size[1]:
             translate = -translate
             scalex = -scalex
@@ -493,8 +507,9 @@ class PreviewCameraX(PreviewCommon, CommonGestures):
             if self._name_pipe:
                 file_id = self._name_pipe[0]
                 self._name_pipe = self._name_pipe[1:]
+        self.capture_in_progress = False
         if self.callback:
-            self.callback(file_id)
+            self.callback(str(file_id))
 
     def _analyze_texture(self):
         if not self.enable_data and self._analyze_callback:
